import argparse
import sys
import sens.image
import sens.status

def main():
    parser = argparse.ArgumentParser(description='Generate twitch status image')
    parser.add_argument('-v', '--verbose', action='store_true', help='verbose')
    parser.add_argument('channel_name', metavar='channel-name', help='Channel to pull data from')
    parser.add_argument('file', help='File to write image to')
    args = parser.parse_args()
    try:
        img = build_image(args.channel_name)
        img.save(args.file, format='png')
    except sens.status.TwitchError as e:
        print('Error retrieving channel data from Twitch.tv', file=sys.stderr)
        if args.verbose:
            print('Response: {json}'.format(json=e.json), file=sys.stderr)

def build_image(channel_name):
    s = sens.status.get_status(channel_name)
    return sens.image.build_image(s)



import urllib.request
import io
from PIL import Image, ImageDraw
ONLINE = 'ONLINE'
OFFLINE = 'OFFLINE'
PADDING = 3

def build_image(twitch_status):
    logo_url = twitch_status.logo_url
    status = twitch_status.status
    name = twitch_status.display_name
    game = twitch_status.game
    online = twitch_status.online
    viewers = twitch_status.viewers
    preview_url = twitch_status.preview_url

    img = Image.new('RGB', (400, 50))
    logo_img = get_image_from_url(logo_url)
    logo_img = logo_img.resize((50, 50))
    img.paste(logo_img, (0,0,50,50))

    draw = ImageDraw.Draw(img)

    font = draw.getfont()
    text_left = logo_img.size[0] + PADDING
    draw.text((text_left,3), status, font=font)

    info_text = '{name} streaming {game}'.format(name=name, game=game)
    draw.text((text_left,18), info_text, font=font)

    if online:
        online_width, _ = font.getsize(ONLINE)
        draw.text((text_left, 33), ONLINE, font=font)

        viewers_left = text_left + online_width + PADDING
        viewers_text = '{viewers} viewers'.format(viewers=viewers)
        draw.text((viewers_left, 33), viewers_text)
        viewers_right = font.getsize(viewers_text)[0] + viewers_left

        preview_max_left = viewers_right + PADDING
        preview_max_size = (img.size[0] - preview_max_left, img.size[1])

        preview_img = get_image_from_url(preview_url)
        preview_img_pos = tuple(map(lambda i: i[0] - i[1], zip(img.size,
            preview_img.size)))
        img.paste(preview_img, preview_img_pos)
    else:
        draw.text((text_left, 33), OFFLINE, font=font)
    return img

def get_image_from_url(url):
    resp = urllib.request.urlopen(url)
    resp_bytes = io.BytesIO(resp.read())
    return Image.open(resp_bytes)



import http.client
import io
import json

HOSTNAME = 'api.twitch.tv'
BASE_PATH = '/kraken'
HEADERS = {'Accept':'application/vnd.twitchtv.v3+json'}

class TwitchError(Exception):
    def __init__(self, response, json_data, *args, **kwargs):
        super(TwitchError, self).__init__(*args, **kwargs)
        self.response = response
        self.json = json_data

class ChannelNotFoundError(TwitchError):
    pass

def get_twitch_path(path):
    conn = http.client.HTTPSConnection(HOSTNAME)
    conn.request('GET', path, headers=HEADERS)
    resp = conn.getresponse()
    json_data = json.load(io.TextIOWrapper(resp))
    if resp.status == 404:
        raise ChannelNotFoundError(resp, json_data)
    elif resp.status != 200:
        raise TwitchError(resp, json_data)
    return json_data

def get_stream(channel_name):
    path = '{base}/streams/{channel_name}'.format(
            base=BASE_PATH, channel_name=channel_name)
    return get_twitch_path(path)

def get_channel(channel_name):
    path = '{base}/channels/{channel_name}'.format(
            base=BASE_PATH, channel_name=channel_name)
    return get_twitch_path(path)

def get_status(channel_name):
    channel_data = get_channel(channel_name)
    stream_data = get_stream(channel_name)
    display_name = channel_data['display_name']
    status = channel_data['status']
    game = channel_data['game']
    logo_url = channel_data['logo']
    online = stream_data['stream'] is not None
    viewers = stream_data['stream']['viewers'] if online else None
    preview_url = stream_data['stream']['preview']['small'] if online else None
    return TwitchStatus(logo_url, status, display_name, game, online,
                        viewers, preview_url)

class TwitchStatus(object):
    def __init__(self, logo_url, status, display_name, game,
                 online, viewers, preview_url):
        self.logo_url = logo_url
        self.status = status
        self.display_name = display_name
        self.game = game
        self.online = online
        self.viewers = viewers
        self.preview_url = preview_url



import setuptools

setuptools.setup(name='Sens',
      version='0.0.2',
      description='A dynamic Twitch status image generator',
      author='Mark McGuire',
      author_email='???',
      url='https://github.com/TronPaul/sens',

      install_requires=['Pillow>=2.2.1'],

      entry_points={
          'console_scripts': ['sens = sens.application:main'],
      },
      packages=setuptools.find_packages(),
      zip_safe=True,
)
